/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#ifndef ATTRIBUTES_H_
#define ATTRIBUTES_H_

#include <map>
#include <boost/any.hpp>

using namespace std;

/**
 * Attribute keys.
 */
enum class Key
{
    BUILDER, MODEL, STRING_COUNT, STYLE, TYPE, TOP_WOOD, BACK_WOOD
};

/**
 * The attributes map: Map an attribute key
 * to an attribute value of any type.
 */
typedef map<Key, boost::any> AttributesMap;

/**
 * The attributes of an instrument specification.
 */
class Attributes
{
public:
    /**
     * Getter.
     * @return the map of attributes.
     */
    AttributesMap get_attributes() { return attributes; }

    /**
     * Get the attribute value given its key.
     * @param key the attribute key.
     * @return the attribute value.
     */
    boost::any get_value(Key key) { return attributes[key]; }

    /**
     * Set the attribute value given its key.
     * @param key the attribute key.
     * @param value the value to set.
     */
    void set(Key key, boost::any value) { attributes[key] = value; }

    /**
     * Test whether this set of attribute values matches
     * another set of attribute values. Compare each
     * value of this set with the corresponding value
     * of the other set.
     * @param other the other set of attribute values.
     * @return true if matches, else false.
     */
    bool matches(Attributes *other);

    /**
     * Print the keys and values of the attributes map.
     */
    void print();

private:
    AttributesMap attributes;  // the attributes map

    /**
     * Test whether the attributes map has a given attribute key.
     * @param key the attribute key.
     * @return true if the map has the key, else false.
     */
    bool has_key(Key key);

    /**
     * Test whether the corresponding attribute value with a given key
     * from another attribute map is equal to this map's value.
     * @param other the other attributes map.
     * @param key the attribute key.
     * @return true if the two attribute values are equal, else false.
     */
    bool equals(Attributes *other, const Key key);
};

/**
 * Overloaded output operator for attribute keys.
 * @param ostr the output stream.
 * @param attr the attribute.
 */
ostream& operator <<(ostream& ostr, const Key attr);

#endif /* ATTRIBUTES_H_ */
